'use strict';

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _React = require('react/addons');

var _React2 = _interopRequireDefault(_React);

var _DocumentMeta = require('../index');

var _DocumentMeta2 = _interopRequireDefault(_DocumentMeta);

var _removeDocumentMeta = require('../utils');

var _getElements$getElement$getAttr = require('./test-utils');

describe('React Document Meta', function () {
  var document = global.document;

  var docMeta = {
    title: 'This is a document title',
    description: 'This meta value is describing the page we are looking at',
    canonical: 'http://domain.tld/path/to/page',
    meta: {
      charset: 'utf-8',
      name: {
        keywords: 'react,document,meta,tags'
      }
    },
    link: {
      rel: {
        stylesheet: ['http://domain.tld/css/vendor.css', 'http://domain.tld/css/styles.css']
      }
    }
  };

  beforeEach(function () {
    _removeDocumentMeta.removeDocumentMeta();
    _React2['default'].addons.TestUtils.renderIntoDocument(_React2['default'].createElement(_DocumentMeta2['default'], docMeta));
  });

  it('should render document.title / <title> according to the title-attr', function () {
    _assert2['default'].strictEqual(document.title, docMeta.title);
  });

  it('should render <meta name="description" content="..."> according to the description-attr', function () {
    _assert2['default'].strictEqual(_getElements$getElement$getAttr.getAttr('meta[name=description]', 'content'), docMeta.description);
  });

  it('should render <link rel="canonical" href="..." according to the canonical-attr', function () {
    _assert2['default'].strictEqual(_getElements$getElement$getAttr.getAttr('link[rel=canonical]', 'href'), docMeta.canonical);
  });

  it('should render simple meta tags, eg. <meta charset="...">', function () {
    _assert2['default'].strictEqual(_getElements$getElement$getAttr.getAttr('meta[charset]', 'charset'), docMeta.meta.charset);
  });

  it('should render normal meta tags, eg. <meta name="..." content="...">', function () {
    Object.keys(docMeta.meta.name).reduce(function (name) {
      _assert2['default'].strictEqual(_getElements$getElement$getAttr.getAttr('meta[name=' + name + ']', 'content'), docMeta.meta.name[name], '<meta name="' + name + '" ... /> has not been rendered correctly');
    });
  });

  it('should render normal link tags, eg. <link rel="..." href="...">', function () {
    Object.keys(docMeta.link.rel).reduce(function (rel) {
      var values = Array.isArray(docMeta.link.rel[rel]) ? docMeta.link.rel[rel] : [docMeta.link.rel[rel]];
      var elements = _getElements$getElement$getAttr.getElements('link[rel=' + rel + ']');
      elements.forEach(function (element, idx) {
        _assert2['default'].strictEqual(element.getAttribute('content'), values[idx], '<link rel="' + rel + '" ... /> has not been rendered correctly');
      });
      //      assert.strictEqual( getAttr(`link[rel=${ rel }]`, 'content'), docMeta.link.rel[rel], `<link rel="${ rel }" ... /> has not been rendered correctly` );
    });
  });
});
