'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { return Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); }

var _React = require('react');

var _React2 = _interopRequireDefault(_React);

var _canUseDOM = require('react/lib/ExecutionEnvironment');

var _createSideEffect = require('react-side-effect');

var _createSideEffect2 = _interopRequireDefault(_createSideEffect);

var _clone$defaults$forEach$isValidProp = require('./utils');

var _meta = {};

function getProps(propsList) {
  var props = {};

  var extend = true;

  for (var i = propsList.length - 1; extend && i >= 0; i--) {
    var _props = _clone$defaults$forEach$isValidProp.clone(propsList[i]);

    if (_props.description) {
      _clone$defaults$forEach$isValidProp.defaults(_props, { meta: { name: { description: _props.description } } });
    }
    if (_props.canonical) {
      _clone$defaults$forEach$isValidProp.defaults(_props, { link: { rel: { canonical: _props.canonical } } });
    }

    _clone$defaults$forEach$isValidProp.defaults(props, _props);
    extend = _props.hasOwnProperty('extend');
  };

  if (props.auto) {
    autoProps(props);
  }

  return props;
}

function autoProps(props) {
  if (props.auto.ograph === true) {
    ograph(props);
  }

  return props;
}

function ograph(p) {
  if (!p.meta) {
    p.meta = {};
  }
  if (!p.meta.property) {
    p.meta.property = {};
  }

  var group = p.meta.property;
  if (group) {
    if (p.title && !group['og:title']) {
      group['og:title'] = p.title;
    }
    if (p.description && !group['og:description']) {
      group['og:description'] = p.description;
    }
    // if ( p.canonical && !group['og:url'] ) {
    //   group['og:url'] = p.canonical;
    // }
  }
  return p;
}

function parseTags(tagName) {
  var props = arguments[1] === undefined ? {} : arguments[1];

  var tags = [];
  var contentKey = tagName === 'link' ? 'href' : 'content';
  Object.keys(props).forEach(function (groupKey) {
    var group = props[groupKey];
    if (typeof group === 'string') {
      tags.push(_defineProperty({
        tagName: tagName }, groupKey, group));
      return;
    }
    Object.keys(group).forEach(function (key) {
      var values = Array.isArray(group[key]) ? group[key] : [group[key]];
      values.forEach(function (value) {
        var _tags$push2;

        tags.push((_tags$push2 = {
          tagName: tagName }, _defineProperty(_tags$push2, groupKey, key), _defineProperty(_tags$push2, contentKey, value), _tags$push2));
      });
    });
  });
  return tags;
}

function getTags(_props) {
  return [].concat(parseTags('meta', _props.meta), parseTags('link', _props.link));
}

function removeNode(node) {
  node.parentNode.removeChild(node);
}

function removeDocumentMeta() {
  _clone$defaults$forEach$isValidProp.forEach(document.querySelectorAll('head [data-rdm]'), removeNode);
}

function insertDocumentMeta(props) {
  removeDocumentMeta();

  _clone$defaults$forEach$isValidProp.forEach(getTags(props), insertDocumentMetaNode);
}

function insertDocumentMetaNode(entry) {
  var tagName = entry.tagName;

  var attr = _objectWithoutProperties(entry, ['tagName']);

  var newNode = document.createElement(tagName);
  for (var prop in attr) {
    if (entry.hasOwnProperty(prop)) {
      newNode.setAttribute(prop, entry[prop]);
    }
  }
  newNode.setAttribute('data-rdm', '');
  document.getElementsByTagName('head')[0].appendChild(newNode);
}

function render(meta, opts) {
  if (typeof opts !== 'object') {
    return meta;
  }

  var i = 0;
  var tags = [];

  function renderTag(entry) {
    var tagName = entry.tagName;

    var attr = _objectWithoutProperties(entry, ['tagName']);

    if (tagName === 'meta') {
      return _React2['default'].createElement('meta', _extends({}, attr, { key: i++, 'data-rdm': true }));
    } else if (tagName === 'link') {
      return _React2['default'].createElement('link', _extends({}, attr, { key: i++, 'data-rdm': true }));
    }

    return null;
  }

  if (meta.title) {
    tags.push(_React2['default'].createElement(
      'title',
      { key: i++ },
      meta.title
    ));
  }

  getTags(meta).reduce(function (acc, entry) {
    tags.push(renderTag(entry));
    return tags;
  }, tags);

  if (opts.asReact) {
    return tags;
  }

  return _React2['default'].renderToStaticMarkup(_React2['default'].createElement(
    'div',
    null,
    tags
  )).replace(/(^<div>|<\/div>$)/g, '');
}

function handleChange(propsList) {
  _meta = getProps(propsList);

  if (_canUseDOM.canUseDOM) {
    document.title = _meta.title || '';
    insertDocumentMeta(_meta);
  }
}

var mixin = {
  displayName: 'DocumentMeta',

  propTypes: {
    title: _React2['default'].PropTypes.string,
    description: _React2['default'].PropTypes.string,
    canonical: _React2['default'].PropTypes.string,
    meta: _React2['default'].PropTypes.objectOf(_React2['default'].PropTypes.oneOfType([_React2['default'].PropTypes.string, _React2['default'].PropTypes.objectOf(_React2['default'].PropTypes.string)])),
    link: _React2['default'].PropTypes.objectOf(_React2['default'].PropTypes.objectOf(_React2['default'].PropTypes.oneOfType([_React2['default'].PropTypes.string, _React2['default'].PropTypes.arrayOf(_React2['default'].PropTypes.string)]))),
    auto: _React2['default'].PropTypes.objectOf(_React2['default'].PropTypes.bool)
  },

  statics: {
    peek: function peek() {
      return _meta;
    },

    rewind: function rewind(opts) {
      var meta = _clone$defaults$forEach$isValidProp.clone(_meta);
      this.dispose();
      return render(meta, opts);
    }
  }
};

var DocumentMeta = _createSideEffect2['default'](handleChange, mixin);

exports['default'] = DocumentMeta;
module.exports = exports['default'];
